﻿

// TestEngine.cs
using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Threading.Channels;

namespace Bird_tool
{
    public delegate bool StepValidator(string response, TestContext context);



    public class TestReport
    {
        public DateTime StartTime { get; set; } = DateTime.Now;
        public DateTime EndTime { get; set; }
        public List<TestReportItem> Items { get; set; } = new List<TestReportItem>();
        public string DeviceInfo { get; set; }
        public string TestResult => Items.All(i => i.Status == TestStatus.Passed) ? "PASS" : "FAIL";
        // 设备信息专用字典
        public Dictionary<string, string> DeviceInfoItems { get; } = new Dictionary<string, string>();
        public string UniqueIdentifier { get; set; } = "report";

        // 添加设备信息项
        public void AddDeviceInfo(string key, string value)
        {
            if (!string.IsNullOrEmpty(value))
            {
                DeviceInfoItems[key] = value;
            }
        }
        // 新增：获取设备信息的键值对
        public IEnumerable<KeyValuePair<string, string>> GetDeviceInfoPairs()
        {
            return DeviceInfoItems;
        }

        // 新增：获取测试项结果
        public IEnumerable<TestReportItem> GetTestItems()
        {
            return Items;
        }
    }

    public class TestReportItem
    {
        public string GroupId { get; set; }
        public string GroupName { get; set; }
        public TestStatus Status { get; set; }
        public List<TestStepResult> StepResults { get; set; } = new List<TestStepResult>();
        public string Details => Status == TestStatus.Passed ? "所有步骤通过" : $"失败步骤: {FailedSteps}";
        public int FailedSteps => StepResults.Count(r => r.Status == TestStatus.Failed);
        public DateTime StartTime { get; set; }
        public DateTime EndTime { get; set; }
    }

    public class TestStepResult
    {
        public string Name { get; set; }
        public TestStatus Status { get; set; } = TestStatus.NotRun;
        public string Details { get; set; } = "";
        public string GroupId { get; set; } = "";
        public string GroupName { get; set; } = "";
        public DateTime StartTime { get; set; }
        public DateTime EndTime { get; set; }
        public TimeSpan Duration => EndTime - StartTime;
        public int RetryCount { get; set; }
        public string ErrorMessage { get; set; } = "";
        
    }

    public enum TestStatus
    {
        NotRun,
        Running,
        Passed,
        Failed,
        Timeout,
        Skipped
    }

    // 步骤运行状态
    public enum StepStatus
    {
        NotRun,
        Running,
        Waiting,
        Failed,
        Success
    }
    public enum ActionMode
    {
        SendCmd,
        SetVal,
        WaitTime,
    }
    public class TestGroup
    {
        // 测试组, 用于多组测试项合并测试
        public string GroupId { get; set; }
        // 测试组 名称 
        public string GroupName { get; set; }
        // 测试组 列名
        public string RowKey { get; set; }
        // 是否显示
        public bool ShowStep { get; set; } = true;

        public static List<TestStepConfig> GetStepsById(List<TestStepConfig> steps, string groupId)
        {
            return steps.Where(step => step.EffectiveGroupId == groupId).ToList();
        }

        // 获取分组信息列表
        // 获取分组信息列表
        public static List<TestGroup> GetGroupInfos(List<TestStepConfig> steps)
        {
            return steps
                .GroupBy(step => step.EffectiveGroupId)
                .Select(g => new TestGroup
                {
                    GroupId = g.Key,
                    GroupName = g.Last().GroupName ?? g.Last().Name, // 默认使用分组ID作为名称
                    RowKey = g.First().RowKey ?? g.Key, // 默认使用分组ID作为列键
                    ShowStep = g.First().ShowStep
                })
                .ToList();
        }


    }


    public class TestStepConfig
    {
        // 新增有效分组ID属性
        public string EffectiveGroupId =>
        string.IsNullOrEmpty(GroupId) ? Key : GroupId;
        // 测试组id, 用于多个测试项合并成一个测试项
        public string GroupId { get; set; }
        // 测试组 名称 
        public string GroupName { get; set; }
        // 测试组 列名
        public string RowKey { get; set; }
        // 基础配置
        public string Name { get; set; }
        public string Details { get; set; }
        // 界面中间显示的引导操作, 或者提示性文字
        public string Tips { get; set; }
        // 任务失败后的提示性文字, 用于提示操作员什么地方异常
        public string FailTips { get; set; }
        // 动作形式
        public ActionMode Action { get; set; } = ActionMode.SendCmd;
        public string Command { get; set; }
        public Func<TestContext, string> CommandGenerator { get; set; }

        public string SuccessPattern { get; set; }
        public string SuccessText { get; set; }
        public string FailurePattern { get; set; }
        public string FailureText { get; set; }
        public string ExtractPattern { get; set; }
        public List<string> VariableNames { get; } = new List<string>();
        // 变量值列表
        public List<string> VariableValues { get; } = new List<string>();

        public int Timeout { get; set; } = 3000;
        // 是否为隐私命令, 隐私命令不显示对应发送命令
        public bool PrivateCammand { get; set; } = false;

        // 失败后重试
        public int MaxRetries { get; set; } = 3;
        public int RetryCount { get; set; } = 0;

        // 是否创建对应的提示列表
        public bool ShowStep { get; set; } = true;

        // 跳转任务失败次数只有在包含 RetryFromKey 时生效
        public string Key { get; set; }
        // 失败后要跳转到的条目
        public string RetryFromKey { get; set; }
        // 失败后是否重试当前步骤
        public bool StepRetry { get; set; } = true;
        public int MaxJump { get; set; } = 3;
        public int JumpCount { get; set; } = 0;

        // 失败后是否继续测试
        public bool FailContinue { get; set; } = false;

        public bool IsDeviceInfoItem { get; set; } = false;
        public string InfoDisplayName { get; set; }
        public string InfoDisplayTemplate { get; set; }

        public string Description { get; set; }

        public int DelayBefore { get; set; } = 0;

        public int DelayAfter { get; set; } = 0;

        public int DelayAfterPrompt { get; set; } = 0;


        // 用户交互
        public bool RequiresUserPrompt { get; set; }
        // 用户确认
        public string PromptQuestion { get; set; }


        public StepStatus stepStatus { get; set; } = StepStatus.NotRun;

        // result data


        // 自定义逻辑
        public StepValidator Validator { get; set; } // 自定义验证函数
        public Action<TestContext> OnSuccess { get; set; }
        public Action<TestContext> OnFailure { get; set; }

        // 自动生成描述（如果未设置）
        public string GetDescription()
        {
            if (!string.IsNullOrEmpty(Description))
                return Description;

            var sb = new StringBuilder();

            if (!string.IsNullOrEmpty(Command))
            {
                sb.AppendLine($"执行命令: {Command}");
            }

            if (!string.IsNullOrEmpty(SuccessPattern))
            {
                sb.AppendLine($"成功匹配: {SuccessPattern}");
            }
            if (!string.IsNullOrEmpty(SuccessText))
            {
                sb.AppendLine($"成功匹配: {SuccessText}");
            }

            if (!string.IsNullOrEmpty(FailurePattern))
            {
                sb.AppendLine($"失败匹配: {FailurePattern}");
            }

            if (!string.IsNullOrEmpty(FailureText))
            {
                sb.AppendLine($"失败匹配: {FailureText}");
            }

            if (RequiresUserPrompt)
            {
                sb.AppendLine($"需要用户确认: {PromptQuestion}");
            }

            if (Validator != null)
            {
                sb.AppendLine("包含自定义验证逻辑");
            }

            return sb.ToString();
        }

    }

    public class TestContext
    {
        public Dictionary<string, string> Variables { get; } = new Dictionary<string, string>();
        public int CurrentStepIndex { get; set; }
        // 重试次数
        public int RetryCount { get; set; }
        // 当前测试项的状态
        public TestStepConfig CurrentStep => Steps?[CurrentStepIndex];
        public List<TestStepConfig> Steps { get; set; }
        public List<TestStepResult> StepResults { get; } = new List<TestStepResult>();
        public TestReport Report { get; set; } = new TestReport();
        public bool IsRunning { get; set; }
    }

    public static class TaskExtensions
    {
        public static async Task<T> WaitAsync<T>(this Task<T> task, CancellationToken cancellationToken)
        {
            var tcs = new TaskCompletionSource<T>();
            using (cancellationToken.Register(() => tcs.TrySetCanceled()))
            {
                return await await Task.WhenAny(task, tcs.Task);
            }
        }
    }

    class TestExecutor
    {

        public delegate void LogHandler(string message, LogLevel level);
        public delegate void LogShowHandler(string message);

        public delegate void PromptHandler(string question, Action onYes, Action onNo, int waitTime = 60);
        public delegate bool StepValidator(string response, TestContext context);
        public delegate void StepChangedHandler(TestStepConfig step, TestContext context, bool isStarting);
        public delegate void TestFailHandler(TestStepConfig step, TestContext context);
        public delegate void TestSuccessHandler(TestContext context);

       


        public event LogHandler OnLog;
        public event LogShowHandler OnLogShow;
        public event PromptHandler OnPrompt;
        public event StepChangedHandler OnStepChanged;
        public event TestFailHandler OnFailed;
        public event TestSuccessHandler OnSuccess;

        private readonly SerialManager _serialManager;
        private TestContext _context;
        private System.Threading.Timer _timeoutTimer;
        private readonly object _lock = new object();
        // 步骤表
        private List<TestStepConfig> _step_map;

        private readonly Channel<Func<CancellationToken, Task>> _taskQueue;
        private CancellationTokenSource _cts = new CancellationTokenSource();
        private Task _queueProcessor;

        public bool isStart { get; private set; } = false;
        public bool isInit { get; private set; } = false;
        public bool enableMakeResult { get; private set; } = false;

        public string ReportFileNameTemplate { get; set; } = "TestReport_{{mac}}_{{timestamp}}";
        // 新增：CSV文件路径
        private string _csvReportPath = "TestReport/report.csv";
        private static readonly object _csvLock = new object();

        public TestExecutor(SerialManager serialManager)
        {
            bird_tool.Log("TestExecutor ");
            _serialManager = serialManager;
            _taskQueue = Channel.CreateBounded<Func<CancellationToken, Task>>(1000);

            StartQueueProcessor();
        }

        private void StartQueueProcessor()
        {
            // 确保之前的队列处理器已停止
            if (_queueProcessor != null && !_queueProcessor.IsCompleted)
            {
                _queueProcessor.Dispose();
            }

            _queueProcessor = Task.Run(async () =>
            {
                while (!_cts.Token.IsCancellationRequested &&
                       await _taskQueue.Reader.WaitToReadAsync(_cts.Token))
                {
                    if (_taskQueue.Reader.TryRead(out var taskFunc))
                    {
                        try
                        {
                            await taskFunc(_cts.Token);
                        }
                        catch (OperationCanceledException)
                        {
                            // 正常取消
                        }
                        catch (Exception ex)
                        {
                            OnLog?.Invoke($"任务执行失败: {ex.Message}", LogLevel.error);
                            OnLogShow?.Invoke($"任务执行失败: {ex.Message}");
                        }
                        
                    }
                }
            });
        }

        private void EnqueueTask(Func<CancellationToken, Task> task)
        {
            if (!_taskQueue.Writer.TryWrite(task))
            {
                OnLog?.Invoke("任务队列已满，无法添加新任务", LogLevel.error);
            }
        }

        public async Task StopTestAsync(bool waitForCompletion = false)
        {
            try
            {
                // 取消所有任务
                _cts.Cancel();

                // 等待队列处理完成（如果需要）
                if (waitForCompletion && _queueProcessor != null && !_queueProcessor.IsCompleted)
                {
                    try
                    {
                        await _queueProcessor;
                    }
                    catch (OperationCanceledException)
                    {
                        // 正常取消
                    }
                }
            }
            catch (Exception ex)
            {
                OnLog?.Invoke($"停止测试时出错: {ex.Message}", LogLevel.error);
            }
            finally
            {
                // 清理资源
                if (_serialManager != null)
                {
                    _serialManager.Disconnect();
                    _serialManager.OnLineReceived -= HandleResponse;
                }

                // 重置状态
                _context = null;
                isStart = false;

                // 重新创建 CTS 以便下次使用
                _cts.Dispose();
                _cts = new CancellationTokenSource();

                // 重新启动队列处理器
                StartQueueProcessor();
            }
        }

        private bool StartTestWithSteps(List<TestStepConfig> steps)
        {
            lock (_lock)
            {
                if (steps == null || steps.Count == 0)
                {
                    OnLog?.Invoke("步骤列表为空，无法启动测试", LogLevel.error);
                    return false;
                }
                foreach (var step in steps)
                {
                    step.JumpCount = 0;
                    step.stepStatus = StepStatus.NotRun;
                    step.RetryCount = 0;
                }
                if (!_serialManager.ReOpen())
                {
                    OnLog?.Invoke("无法打开串口", LogLevel.error);
                    OnLogShow?.Invoke("串口打开失败");
                }
                _serialManager.OnLineReceived += HandleResponse;
                EnqueueTask(async ct =>
                {
                    // 初始化上下文
                    _context = new TestContext
                    {
                        Steps = steps, // 使用传入的步骤列表
                        CurrentStepIndex = -1,
                        Report = new TestReport(),
                        IsRunning = true,
                    };
                    isStart = true;
                    MoveToNextStep();

                });

                return true;
            }
        }

        public bool StartTest()
        {
            lock (_lock)
            {
                if (!isInit)
                {
                    return false;
                }
                enableMakeResult = true;
                return StartTestWithSteps(_step_map); ;
            }
        }
        public bool StartTestFromGroup(string groupId)
        {
            lock (_lock)
            {
                if (!isInit)
                {
                    return false;
                }

                // 获取分组的所有步骤
                int firstIndex = _step_map.FindIndex(step => step.EffectiveGroupId == groupId);
                if (firstIndex == -1)
                {
                    OnLog?.Invoke($"无法找到分组 {groupId} 的起始步骤", LogLevel.error);
                    return false;
                }
                // todo 从第一个id出现的位置提取分组后面的所有内容
                var stepsFromGroup = _step_map.Skip(firstIndex).ToList();
                foreach (var step in stepsFromGroup)
                {
                    step.JumpCount = 0;
                    step.stepStatus = StepStatus.NotRun;
                    step.RetryCount = 0;
                }
                enableMakeResult = false;
                return StartTestWithSteps(stepsFromGroup);
            }
        }
        public bool StartTestGroup(string groupId)
        {
            lock (_lock)
            {
                if (!isInit)
                {
                    return false;
                }

                // 直接从 _step_map 中提取分组步骤
                var groupSteps = TestGroup.GetStepsById(_step_map, groupId);
                if (groupSteps.Count == 0)
                {
                    OnLog?.Invoke($"未找到分组ID为 {groupId} 的步骤", LogLevel.error);
                    return false;
                }
                OnLog?.Invoke($"找到分组ID为 {groupId} 的步骤", LogLevel.info);

                enableMakeResult = false;
                // 使用提取的分组步骤启动测试
                return StartTestWithSteps(groupSteps);
            }
        }


        public bool InitTest(List<TestStepConfig> steps)
        {
            if (steps.Count < 1)
            {
                return false;
            }
            lock (_lock)
            {

                _step_map = InitSteps(steps);

                isStart = false;
                isInit = true;
                return true;
            }
        }
        public List<TestStepConfig> InitSteps(List<TestStepConfig> steps)
        {
            var keySet = new HashSet<string>();
            int counter = 1;

            // 创建分组字典，用于存储分组信息
            var groupInfoDict = new Dictionary<string, (string GroupName, string RowKey)>();

            foreach (var step in steps)
            {
                // 确保每个步骤都有唯一的Key
                if (string.IsNullOrEmpty(step.Key))
                {
                    step.Key = $"step_{counter++}";
                }
                else if (keySet.Contains(step.Key))
                {
                    int suffix = 2;
                    string newKey;
                    do
                    {
                        newKey = $"{step.Key}_{suffix++}";
                    } while (keySet.Contains(newKey));

                    step.Key = newKey;
                    OnLog?.Invoke($"警告: 步骤Key '{step.Key}' 已存在，已重命名为 '{newKey}'", LogLevel.error);
                }
                keySet.Add(step.Key);

                // 处理分组信息
                if (string.IsNullOrEmpty(step.GroupId))
                {
                    // 如果没有分组ID，使用Key作为默认分组ID
                    step.GroupId = step.Key;
                    step.GroupName = step.Name;
                    step.RowKey = step.GroupId;
                }
            }

            // 第二遍遍历：按分组处理
            var groupedSteps = steps
                .GroupBy(s => s.GroupId)
                .ToList();

            foreach (var group in groupedSteps)
            {
                var groupId = group.Key;
                var groupSteps = group.ToList();

                // 确定分组名：
                // 1. 优先使用组内已设置的GroupName
                // 2. 如果都没有设置，使用最后一个步骤的Name
                string groupName = groupSteps
                    .Where(s => !string.IsNullOrEmpty(s.GroupName))
                    .Select(s => s.GroupName)
                    .FirstOrDefault();

                if (string.IsNullOrEmpty(groupName))
                {
                    groupName = groupSteps.Last().Name;
                }

                // 确定行键：
                // 1. 优先使用组内已设置的RowKey
                // 2. 如果都没有设置，使用分组ID
                string rowKey = groupSteps
                    .Where(s => !string.IsNullOrEmpty(s.RowKey))
                    .Select(s => s.RowKey)
                    .FirstOrDefault();

                if (string.IsNullOrEmpty(rowKey))
                {
                    rowKey = groupId;
                }


                // 应用分组信息到所有步骤
                foreach (var step in groupSteps)
                {
                    step.GroupName = groupName;
                    step.RowKey = rowKey;
                }
            }

            return steps;
        }


        // 获取所有分组ID
        public List<string> GetAllGroupIds()
        {
            lock (_lock)
            {
                return _step_map?
                    .Select(step => step.EffectiveGroupId)
                    .Distinct()
                    .ToList() ?? new List<string>();
            }
        }

        // 获取分组信息列表
        public List<TestGroup> GetGroupInfos()
        {
            lock (_lock)
            {
                return _step_map?
                    .GroupBy(step => step.EffectiveGroupId)
                    .Select(g => new TestGroup
                    {
                        GroupId = g.Key,
                        GroupName = !string.IsNullOrWhiteSpace(g.First().GroupName)
                            ? g.First().GroupName
                            : g.First().Name,
                        RowKey = !string.IsNullOrWhiteSpace(g.First().RowKey)
                                ? g.First().RowKey
                                : g.Key // 默认使用分组ID作为列键
                    })
                    .ToList() ?? new List<TestGroup>();
            }
        }

        // 字符串模板替换方法
        public string ReplaceTemplateVariables(string input, Dictionary<string, string> variables)
        {
            if (string.IsNullOrWhiteSpace(input))
                return input;

            // 创建大小写不敏感的合并字典
            var mergedVars = new Dictionary<string, string>(StringComparer.OrdinalIgnoreCase);

            // 1. 添加用户提供的变量（不修改原始字典）
            if (variables != null)
            {
                foreach (var kv in variables)
                {
                    mergedVars[kv.Key] = kv.Value;
                }
            }

            // 2. 添加系统变量（仅当用户未定义时）
            AddIfMissing(mergedVars, "timestamp", DateTime.Now.ToString("yyyyMMddHHmmss"));
            AddIfMissing(mergedVars, "date", DateTime.Now.ToString("yyyyMMdd"));
            AddIfMissing(mergedVars, "time", DateTime.Now.ToString("HHmmss"));
            AddIfMissing(mergedVars, "datetime", DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"));
            AddIfMissing(mergedVars, "machine", Environment.MachineName);
            AddIfMissing(mergedVars, "user", Environment.UserName);

            return Regex.Replace(input, @"\{\{(\w+)\}\}", match =>
            {
                var varName = match.Groups[1].Value;

                // 尝试各种格式的键名匹配
                if (TryGetValueIgnoreCase(mergedVars, varName, out var value))
                    return value;

                // 尝试下划线格式（如 device_model）
                var underscored = Regex.Replace(varName, @"([a-z])([A-Z])", "$1_$2").ToLower();
                if (TryGetValueIgnoreCase(mergedVars, underscored, out value))
                    return value;

                // 尝试空格格式（如 Device Model）
                var spaced = Regex.Replace(varName, @"([a-z])([A-Z])", "$1 $2");
                if (TryGetValueIgnoreCase(mergedVars, spaced, out value))
                    return value;

                // 尝试驼峰格式（如 deviceModel）
                var camelCase = char.ToLower(varName[0]) + varName.Substring(1);
                if (TryGetValueIgnoreCase(mergedVars, camelCase, out value))
                    return value;

                return $"{{{{UNDEFINED:{varName}}}}}";
            });
        }
        // 辅助方法：仅在键不存在时添加
        private void AddIfMissing(Dictionary<string, string> dict, string key, string value)
        {
            if (!dict.ContainsKey(key))
            {
                dict[key] = value;
            }
        }

        // 辅助方法：忽略大小写获取值
        private bool TryGetValueIgnoreCase(Dictionary<string, string> dict, string key, out string value)
        {
            // 直接匹配
            if (dict.TryGetValue(key, out value))
                return true;

            // 大小写不敏感匹配
            var keyComparer = StringComparer.OrdinalIgnoreCase;
            foreach (var kv in dict)
            {
                if (keyComparer.Equals(kv.Key, key))
                {
                    value = kv.Value;
                    return true;
                }
            }

            value = null;
            return false;
        }

        private async Task ExecuteCurrentStepAsync(CancellationToken ct)
        {
            var step = _context.CurrentStep;
            RecordStepResult(step, TestStatus.Running, "步骤开始执行");
            // 记录步骤开始
            OnLog?.Invoke($"开始步骤: {step.Name}", LogLevel.info);
            step.stepStatus = StepStatus.Running;
            OnStepChanged?.Invoke(step, _context, true);
            if (step.DelayBefore > 0)
            {
                OnLog?.Invoke($"等待 {step.DelayBefore}ms (执行前延迟)", LogLevel.debug);
                await Task.Delay(step.DelayBefore);
            }

            if (step.Action == ActionMode.SetVal)
            {
                // 从VaruableValues 将内容设置至 VaruableNames 中
                if (step.VariableValues.Count == step.VariableNames.Count)
                {
                    for (int i = 0; i < step.VariableValues.Count; i++)
                    {
                        var varName = step.VariableNames[i];
                        string tmp_str = $"设置变量: {varName} = {step.VariableValues[i]}";
                        _context.Variables[varName] = step.VariableValues[i];
                        if (step.PrivateCammand)
                        {
                            OnLog?.Invoke("设置变量", LogLevel.debug);
                        }
                        else
                        {
                            OnLog?.Invoke(tmp_str, LogLevel.info);
                        }
                    }
                    HandleSetpSuccess("变量设置完成");
                }
                else
                {
                    HandleStepFailure("配置文件异常, 变量长度异常");
                }
            }
            else if (step.Action == ActionMode.SendCmd)
            {
                // 重置超时计时器
                _timeoutTimer?.Dispose();
                _timeoutTimer = null;

                var command = step.Command;
                // 支持动态命令生成
                if (step.CommandGenerator != null)
                {
                    command = step.CommandGenerator(_context);
                }

                // 新增模板变量替换
                command = ReplaceTemplateVariables(command, _context.Variables);

                // 发送命令
                if (!string.IsNullOrEmpty(command))
                {
                    OnLog?.Invoke($"发送命令: {command.Trim()}", step.PrivateCammand? LogLevel.debug: LogLevel.info);
                    _serialManager.SendCommand(command);
                }

                if (step.DelayAfter > 0)
                {
                    OnLog?.Invoke($"等待 {step.DelayBefore}ms (发送后延迟)", LogLevel.debug);
                    await Task.Delay(step.DelayAfter);
                }
                // 让当前任务的状态转变未Waiting, 用于在发送消息在进行匹配
                step.stepStatus = StepStatus.Waiting;

                // 重构响应等待逻辑
                var timeoutCts = new CancellationTokenSource(step.Timeout);
                var linkedCts = CancellationTokenSource.CreateLinkedTokenSource(ct, timeoutCts.Token) ;
                OnLog?.Invoke($"发送命令 test",  LogLevel.info);
                try
                {
                    if (step.RequiresUserPrompt &&
                        string.IsNullOrEmpty(step.SuccessPattern) &&
                        string.IsNullOrEmpty(step.SuccessText))
                    {
                        await PromptQuestionAsync(step, linkedCts.Token);
                    }
                    else
                    {
                        OnLog?.Invoke($"等待响应(发送后延迟)", LogLevel.debug);
                        var response = await WaitForResponseAsync(step, linkedCts.Token);
                        ProcessResponse(response, step, linkedCts.Token);
                    }
                }
                catch (OperationCanceledException) when (timeoutCts.IsCancellationRequested)
                {
                    HandleStepFailure("操作超时", false);
                }

                // 如果没有SuccessPattern 与 SuccessText则直接进行询问
                if (step.RequiresUserPrompt && string.IsNullOrEmpty(step.SuccessPattern) && string.IsNullOrEmpty(step.SuccessText))
                {
                   await PromptQuestionAsync(step, linkedCts.Token);
                }
            }

        }
        private async Task<string> WaitForResponseAsync(TestStepConfig step, CancellationToken ct)
        {
            var tcs = new TaskCompletionSource<string>();
            Action<string> responseHandler = (data) =>
            {
                if ((!string.IsNullOrEmpty(step.SuccessPattern) && Regex.IsMatch(data, step.SuccessPattern)) ||
                    (!string.IsNullOrEmpty(step.SuccessText) && data.Contains(step.SuccessText)) ||
                    (!string.IsNullOrEmpty(step.FailurePattern) && Regex.IsMatch(data, step.FailurePattern)) ||
                    (!string.IsNullOrEmpty(step.FailureText) && data.Contains(step.FailureText)))
                {
                    tcs.TrySetResult(data);
                }
            };

            try
            {
                _serialManager.OnLineReceived += responseHandler;
                return await tcs.Task.WaitAsync(ct);
            }
            finally
            {
                _serialManager.OnLineReceived -= responseHandler;
            }
        }


        // 处理响应
        private async void ProcessResponse(string data, TestStepConfig step, CancellationToken ct)
        {
            if ((!string.IsNullOrEmpty(step.SuccessPattern) && Regex.IsMatch(data, step.SuccessPattern)) ||
                (!string.IsNullOrEmpty(step.SuccessText) && data.Contains(step.SuccessText)))
            {
                if (!string.IsNullOrEmpty(step.ExtractPattern) && !ExtractVariables(data, step))
                {
                    if (!ExtractVariables(data, step))
                    {
                        OnLog?.Invoke($"匹配成功, 但是无法提取变量 {data} ", LogLevel.error);
                        return;
                    }
                }

                // 自定义验证逻辑
                bool shouldRetry = false;
                if (step.Validator != null)
                {
                    shouldRetry = step.Validator(data, _context);
                }

                if (shouldRetry)
                {
                    OnLog?.Invoke("验证失败，需要重试", LogLevel.info);
                    HandleStepFailure("自定义验证失败", false);
                }
                else
                {
                    // 判断是否需要进行提问. 如果需要提问则先进行提问
                    if (step.RequiresUserPrompt)
                    {
                        await PromptQuestionAsync(step, ct);
                    }
                    else
                    {
                        HandleSetpSuccess("匹配到成功关键词");
                    }
                }
            }
            else if ((!string.IsNullOrEmpty(step.FailurePattern) && Regex.IsMatch(data, step.FailurePattern)) ||
                     (!string.IsNullOrEmpty(step.FailureText) && data.Contains(step.FailureText)))
            {
                HandleStepFailure("匹配到失败关键词", false);
            }
        }

        public void HandleResponse(string data)
        {
            OnLog?.Invoke($"收到响应 {data} ", LogLevel.debug);
        }

        private bool ExtractVariables(string data, TestStepConfig step)
        {
            if (string.IsNullOrEmpty(step.ExtractPattern)) return false;
            bool isMatch = false;
            int matchTotal = 0;
            var match = Regex.Match(data, step.ExtractPattern);
            if (match.Success)
            {
                for (int i = 1; i < match.Groups.Count; i++)
                {
                    var varName = step.VariableNames.Count >= i ?
                        step.VariableNames[i - 1] : $"var{i - 1}";
                    string msg = $"提取变量: {varName} = {match.Groups[i].Value}";
                    _context.Variables[varName] = match.Groups[i].Value;
                    matchTotal++;
                    isMatch = true;
                    OnLog?.Invoke(msg, step.PrivateCammand? LogLevel.debug: LogLevel.info);
                }
            }
            // 记录缺失变量
            if (step.VariableNames.Count > match.Groups.Count - 1)
            {
                OnLog?.Invoke($"警告: 提取变量不足(需要{step.VariableNames.Count} 实际{match.Groups.Count - 1})",
                              LogLevel.error);
            }
            return isMatch;

        }

        private void HandleTestFail()
        {
            HandleTestEnd(false);
            OnFailed?.Invoke(_context.CurrentStep, _context);
            StopTestAsync();
        }
        private void HandleTestSuccess()
        {
            HandleTestEnd(true);
            OnSuccess?.Invoke(_context);
            StopTestAsync();
        }

        // 替换现有的 PromptQuestion 方法
        private async Task PromptQuestionAsync(TestStepConfig step, CancellationToken ct)
        {
            OnLog?.Invoke($"等待用户确认: {step.PromptQuestion}", LogLevel.info);

            if (step.DelayAfterPrompt > 0)
            {
                OnLog?.Invoke($"等待 {step.DelayAfterPrompt}ms (提示延迟)", LogLevel.debug);
                await Task.Delay(step.DelayAfterPrompt, ct);
            }

            var promptTcs = new TaskCompletionSource<bool>();

            // 在主线程执行UI提示
            OnPrompt?.Invoke(
                ReplaceTemplateVariables(step.PromptQuestion, _context.Variables),
                () => promptTcs.TrySetResult(true),
                () => promptTcs.TrySetResult(false)
            );

            try
            {
                bool result = await promptTcs.Task.WaitAsync(ct);

                if (result)
                    HandleSetpSuccess("用户确认成功");
                else
                    HandleStepFailure("用户确认失败", true);
            }
            catch (OperationCanceledException)
            {
                HandleStepFailure("用户响应超时", true);
            }
        }

        private void HandleSetpSuccess(string message)
        {
            _timeoutTimer?.Dispose();
            _timeoutTimer = null;
            var step = _context.CurrentStep;
            step.stepStatus = StepStatus.Success;
            RecordStepResult(step, TestStatus.Passed, message);
            // 捕获设备信息
            if (step.IsDeviceInfoItem)
            {
                // 使用用户指定的显示名称或默认使用步骤名
                var infoKey = !string.IsNullOrEmpty(step.InfoDisplayName)
                    ? step.InfoDisplayName
                    : step.Name;

                // 尝试从变量中提取信息值
                string infoValue = "";

                // 情况1：步骤提取了变量
                if (step.VariableNames.Count > 0 &&
                    _context.Variables.TryGetValue(step.VariableNames[0], out var varValue))
                {
                    infoValue = varValue;
                }
                // 情况2：使用成功消息
                else if (!string.IsNullOrEmpty(message))
                {
                    infoValue = message;
                }
                // 判断是否有 InfoDisplayTemplate 输出模板
                if (!string.IsNullOrEmpty(step.InfoDisplayTemplate))
                {
                    infoValue = ReplaceTemplateVariables(step.InfoDisplayTemplate, _context.Variables);
                }

                _context.Report.AddDeviceInfo(infoKey, infoValue);
            }

            RecordStepResult(step, TestStatus.Passed, message);

            OnStepChanged?.Invoke(step, _context, false);
            OnLog?.Invoke($"✅ 步骤完成: {step.Name} ({message})", LogLevel.info);
            MoveToNextStep();
        }
        // 修改HTML生成方法，添加设备信息展示
        private void SaveReportToHtml(TestReport report)
        {
            // 生成设备信息表格
            string deviceInfoHtml = "";
            if (report.DeviceInfoItems.Count > 0)
            {
                deviceInfoHtml = $@"
                <h2>设备信息</h2>
                <table class='device-info'>
                    {string.Join("\n", report.DeviceInfoItems.Select(kv => $@"
                    <tr>
                        <td><strong>{kv.Key}</strong></td>
                        <td>{kv.Value}</td>
                    </tr>"))}
                </table>";
                }

                var html = $@"
                <!DOCTYPE html>
                <html>
                <head>
                    <title>设备测试报告</title>
                    <style>
                        /* ... 现有样式 ... */
                        .device-info td:first-child {{ width: 30%; font-weight: bold; }}
                        .device-info td {{ vertical-align: top; padding: 6px 8px; }}
                    </style>
                </head>
                <body>
                    <h1>设备测试报告</h1>
                    <p><strong>测试结果:</strong> <span class='{report.TestResult.ToLower()}'>{
                            report.TestResult}</span></p>
                    <p><strong>开始时间:</strong> {report.StartTime:yyyy-MM-dd HH:mm:ss}</p>
                    <p><strong>结束时间:</strong> {report.EndTime:yyyy-MM-dd HH:mm:ss}</p>
            
               
                    {deviceInfoHtml}
            
                    <h2>测试项汇总</h2>
                    <table>
                        <tr>
                            <th>测试项</th>
                            <th>状态</th>
                            <th>开始时间</th>
                            <th>结束时间</th>
                            <th>详情</th>
                        </tr>
                        {string.Join("\n", report.Items.Select(i => $@"
                        <tr class='{i.Status.ToString().ToLower()}'>
                            <td>{i.GroupName}</td>
                            <td>{i.Status}</td>
                            <td>{i.StartTime:HH:mm:ss}</td>
                            <td>{i.EndTime:HH:mm:ss}</td>
                            <td>{i.Details}</td>
                        </tr>"))}
                    </table>
            
                    <h2>详细步骤</h2>
                    <table>
                        <tr>
                            <th>步骤名称</th>
                            <th>所属测试项</th>
                            <th>状态</th>
                            <th>开始时间</th>
                            <th>结束时间</th>
                            <th>详情</th>
                        </tr>
                        {string.Join("\n", report.Items.SelectMany(i => i.StepResults.Select(s => $@"
                        <tr class='{s.Status.ToString().ToLower()}'>
                            <td>{s.Name}</td>
                            <td>{i.GroupName}</td>
                            <td>{s.Status}</td>
                            <td>{s.StartTime:HH:mm:ss}</td>
                            <td>{s.EndTime:HH:mm:ss}</td>
                            <td>{s.Details}</td>
                        </tr>")))}
                    </table>
                </body>
                </html>";
            Directory.CreateDirectory("TestReport");
            var fileName = GetSafeFileName("TestReport");
            File.WriteAllText(fileName, html);
            string str = $"测试报告已生成: {Path.GetFullPath(fileName)}";
            OnLog?.Invoke(str, LogLevel.info);
            OnLogShow?.Invoke(str);
            
        }

        public string GetSafeFileName(string dir, string extension = "html")
        {
            // 移除非法字符
            var invalidChars = Path.GetInvalidFileNameChars();
            var safeName = ReplaceTemplateVariables(ReportFileNameTemplate, _context.Variables);

            // 替换空格
            // 替换空格和特殊字符
            safeName = safeName.Replace(" ", "_")
                               .Replace(":", "-")
                               .Replace("/", "-");
            
            // 确保长度合理
            if (safeName.Length > 100)
                safeName = safeName.Substring(0, 100);

            // 确保非空
            if (string.IsNullOrWhiteSpace(safeName))
                safeName = "TestReport";

            return $"{dir}/{safeName}.{extension}";
        }

        private void RecordStepResult(TestStepConfig step, TestStatus status, string message = "")
        {
            var result = new TestStepResult
            {
                Name = step.Name,
                Status = status,
                Details = message,
                StartTime = DateTime.Now,
                EndTime = DateTime.Now,
                GroupId = step.EffectiveGroupId,
                GroupName = step.GroupName
            };

            _context.StepResults.Add(result);
        }

        private void GenerateTestReport()
        {
            
            // 按GroupId分组聚合结果
            var groupedResults = _context.StepResults
                .Where(r => !string.IsNullOrEmpty(r.GroupId))
                .GroupBy(r => r.GroupId)
                .Select(g => new TestReportItem
                {
                    GroupId = g.Key,
                    GroupName = g.First().GroupName,
                    Status = g.Any(r => r.Status == TestStatus.Failed) ?
                             TestStatus.Failed : TestStatus.Passed,
                    StepResults = g.ToList(),
                    StartTime = g.Min(r => r.StartTime),
                    EndTime = g.Max(r => r.EndTime)
                })
                .ToList();

            // 添加未分组步骤
            var ungroupedResults = _context.StepResults
                .Where(r => string.IsNullOrEmpty(r.GroupId))
                .Select(r => new TestReportItem
                {
                    GroupId = "single_" + r.Name,
                    GroupName = r.Name,
                    Status = r.Status,
                    StepResults = new List<TestStepResult> { r },
                    StartTime = r.StartTime,
                    EndTime = r.EndTime
                });

            _context.Report.Items.AddRange(groupedResults);
            _context.Report.Items.AddRange(ungroupedResults);
            _context.Report.EndTime = DateTime.Now;
        }
        // 在测试结束时调用生成报告
        private void HandleTestEnd(bool saveReport)
        {
            if (saveReport && enableMakeResult)
            {
                GenerateTestReport();
                SaveReportToHtml(_context.Report);
                AppendReportToCsv(_context.Report); // 新增CSV记录
            }
        }

        private void HandleStepFailure(string message, bool isQuestion = false)
        {
            _timeoutTimer?.Dispose();
            _timeoutTimer = null;
            if (_context == null)
            {
                OnLog?.Invoke($"测试异常 {message}", LogLevel.error);
                HandleTestFail();
                return;
            }
            var step = _context.CurrentStep;
            step.stepStatus = StepStatus.Failed;
            step.RetryCount++;
            string failStr = $"【{step.Name}】失败 {message} (重试 {step.RetryCount}/{step.MaxRetries})";
            
            OnLog?.Invoke(failStr, LogLevel.info);
            RecordStepResult(step, TestStatus.Failed, message);
            LogLevel level = LogLevel.info;
            // 判断是跳转到特定任务继续执行还是走失败的逻辑
            bool isJump = false;
            int jumpIdx = -1;
            bool allowRetry = false;
            // 判断当前步骤是否含有跳转执行的机会
            if (!string.IsNullOrEmpty(step.RetryFromKey))
            {
                isJump = true;
                if (step.JumpCount < step.MaxJump)
                {
                    jumpIdx = FindKeyByIndex(step.RetryFromKey);
                    if (jumpIdx < 0)
                    {
                        failStr = $"【{step.Name}】 无法找到要跳转的任务【{step.RetryFromKey}】, 请检查配置";
                        level = LogLevel.error;
                        OnLog?.Invoke(failStr, level);
                    }
                }
                else
                {
                    // 需要跳转, 但是已经没有机会了
                    failStr = $"【{step.Name}】任务多次跳转执行失败";
                    level = LogLevel.error;
                }
            }


            // 判断是否需要执行当前的步骤
            if (step.StepRetry)
            {
                // 判断是否已经超出次数
                if (step.RetryCount >= step.MaxRetries)
                {
                    failStr = $"【{step.Name}】 {message} (超过最大重试次数)";
                    level = LogLevel.info;
                    if (isJump && jumpIdx >= 0)
                    {
                        OnLog?.Invoke(
                            $"【{step.Name}】 子步骤失败: {step.Name} 将跳转至【{step.RetryFromKey}】重新执行",
                            LogLevel.info
                            );
                    }
                }
                else
                {
                    allowRetry = true;
                }
            }

            EnqueueTask(async ct =>
            {
                if (allowRetry)
                {
                    // 重新执行任务
                    await ExecuteCurrentStepAsync(ct);
                }
                else if (isJump && jumpIdx >= 0)
                {
                    // 跳转至特定任务重新执行
                    JumpToStep(jumpIdx);
                }
                else
                {
                    _context.IsRunning = false;
                    OnLog?.Invoke(failStr, level);
                    // 自定义失败处理
                    step.OnFailure?.Invoke(_context);
                    OnStepChanged?.Invoke(step, _context, false);
                    // 测试失败, 判断是直接失败还是忽略此项
                    if (step.FailContinue)
                    {
                        MoveToNextStep();
                    }
                    else
                    {
                        HandleTestFail();
                    }
                }
            });
        }

        private void MoveToNextStep()
        {
            EnqueueTask(async ct =>
            {
                // 确保上下文存在
                if (_context == null || !_context.IsRunning)
                {
                    OnLog?.Invoke($"========== 程序未执行,无法进入下一步 ==========", LogLevel.error);
                    OnLogShow?.Invoke($"========== 程序未执行,无法进入下一步 ==========");
                    return;
                }
                _context.CurrentStepIndex++;

                if (_context.CurrentStepIndex >= _context.Steps.Count)
                {
                    HandleTestSuccess();
                    return;
                }

                OnStepChanged?.Invoke(_context.CurrentStep, _context, true);
                OnLog?.Invoke($"========== 进入步骤: {_context.CurrentStep.Name} ==========", LogLevel.info);

                await ExecuteCurrentStepAsync(ct);
            });
        }
        private int FindKeyByIndex(string key)
        {
            for (int i = 0; i < _context.Steps.Count; i++)
            {
                var step = _context.Steps[i];
                if (step.Key == key)
                {
                    return i;
                }
            }
            return -1;
        }
        
        private void JumpToStep(int newIndex)
        {
            EnqueueTask(async ct =>
            {
                if (newIndex < 0 || newIndex >= _context.Steps.Count)
                {
                    OnLog?.Invoke($"无效的步骤索引: {newIndex}", LogLevel.error);
                    return;
                }
                // 跳转计数器增加
                _context.CurrentStep.JumpCount++;
                OnLog?.Invoke($"跳转到步骤: {_context.Steps[newIndex].Name}", LogLevel.debug);
                // 当前步骤的失败保留, 前面任务的失败进行移除
                int oldIdx = _context.CurrentStepIndex;
                if (oldIdx >= 0)
                {
                    if (oldIdx < newIndex)
                    {
                        // 跳转到后方, 一般不会出现
                        for (int i = oldIdx; i <= newIndex; i++)
                        {
                            var step = _context.Steps[i];
                            step.RetryCount = 0;
                        }

                    }
                    else if (newIndex < oldIdx)
                    {
                        // 跳转到前方执行
                        for (int i = newIndex; i < oldIdx; i++)
                        {
                            var step = _context.Steps[i];
                            step.RetryCount = 0;
                        }
                    }
                }

                _context.CurrentStepIndex = newIndex;
                await ExecuteCurrentStepAsync(ct);
            });
        }


        // 新增：追加CSV记录
        private void AppendReportToCsv(TestReport report)
        {
            lock (_csvLock)
            {
                bool fileExists = File.Exists(_csvReportPath);

                using (var writer = new StreamWriter(_csvReportPath, true, Encoding.UTF8))
                {
                    // 写入表头（如果文件不存在）
                    if (!fileExists)
                    {
                        var headers = new List<string>
                    {
                        "StartTime", "EndTime", "Result"
                    };

                        // 添加设备信息列
                        foreach (var kv in report.GetDeviceInfoPairs())
                        {
                            headers.Add(kv.Key);
                        }

                        // 添加测试项列
                        foreach (var item in report.GetTestItems())
                        {
                            headers.Add($"{item.GroupName}_Status");
                            headers.Add($"{item.GroupName}_Details");
                        }

                        writer.WriteLine(string.Join(",", headers));
                    }

                    // 构建数据行
                    var rowData = new List<string>
                {
                    report.StartTime.ToString("yyyy-MM-dd HH:mm:ss"),
                    report.EndTime.ToString("yyyy-MM-dd HH:mm:ss"),
                    report.TestResult
                };

                    // 添加设备信息值
                    foreach (var kv in report.GetDeviceInfoPairs())
                    {
                        rowData.Add(kv.Value);
                    }

                    // 添加测试项结果
                    foreach (var item in report.GetTestItems())
                    {
                        rowData.Add(item.Status.ToString());
                        rowData.Add(item.Details.Replace(",", ";"));
                    }

                    writer.WriteLine(string.Join(",", rowData));
                }
            }
        }


    }
}