﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace Bird_tool
{
    public class TestStepList : UserControl
    {
        // 添加按钮事件委托
        public delegate void StepButtonClickHandler(string key, TestType testType);
        public event StepButtonClickHandler OnStepButtonClick;

        // 步骤项数据结构
        public class TestStep
        {
            public string Key { get; set; }
            public string Name { get; set; }
            public StepStatus Status { get; set; }
            public DateTime? StartTime { get; set; }
            public DateTime? EndTime { get; set; }
            public Button BtnStartFrom { get; set; }
            public Button BtnTestSingle { get; set; }
        }


        // 控件
        private FlowLayoutPanel listContainer;
        private List<TestStep> steps = new List<TestStep>();
        private Dictionary<string, Panel> stepControls = new Dictionary<string, Panel>();
        private bool isTestRunning = false; // 测试运行状态标志

        // 状态颜色
        private static readonly Color PendingColor = Color.LightGray;
        private static readonly Color RunningColor = Color.DodgerBlue;
        private static readonly Color SuccessColor = Color.LimeGreen;
        private static readonly Color WaitingColor = Color.Gold;
        private static readonly Color FailedColor = Color.OrangeRed;

        // 图标
        private static readonly Image PendingIcon = CreateStatusIcon(PendingColor);
        private static readonly Image RunningIcon = CreateStatusIcon(RunningColor);
        private static readonly Image SuccessIcon = CreateStatusIcon(SuccessColor);
        private static readonly Image WaitingIcon = CreateStatusIcon(WaitingColor);
        private static readonly Image FailedIcon = CreateStatusIcon(FailedColor);


        public TestStepList()
        {
            SetupUI();
        }

        private void SetupUI()
        {
            this.Dock = DockStyle.Fill;
            this.Padding = new Padding(5);
            this.BackColor = Color.White;
            this.BorderStyle = BorderStyle.FixedSingle;

            // 主容器 - 垂直滚动列表
            listContainer = new FlowLayoutPanel
            {
                Dock = DockStyle.Fill,
                FlowDirection = FlowDirection.TopDown,
                WrapContents = false,
                AutoScroll = true,
                BackColor = Color.White,
                AutoScrollMargin = new Size(0, 5)
            };

            this.Controls.Add(listContainer);
        }

        // 添加测试步骤
        public void AddStep(string key, string name)
        {
            var step = new TestStep
            {
                Key = key,
                Name = name,
                Status = StepStatus.NotRun
            };

            steps.Add(step);
            AddStepToUI(step);
        }

        // 更新步骤状态
        public void UpdateStepStatus(string key, string name, StepStatus status)
        {
            var step = steps.Find(s => s.Key == key);
            if (step != null)
            {
                step.Status = status;

                if (status == StepStatus.Running && !step.StartTime.HasValue)
                {
                    step.StartTime = DateTime.Now;
                }
                else if ((status == StepStatus.Success || status == StepStatus.Failed ||
                         status == StepStatus.Waiting) && !step.EndTime.HasValue)
                {
                    step.EndTime = DateTime.Now;
                }

                UpdateStepUI(step);
            }
        }

        // 设置测试运行状态
        public void SetTestRunning(bool running)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new Action(() => SetTestRunning(running)));
                return;
            }

            isTestRunning = running;
            UpdateButtonsVisibility();
        }

        // 更新按钮可见性
        private void UpdateButtonsVisibility()
        {
            foreach (var step in steps)
            {
                if (step.BtnStartFrom != null)
                    step.BtnStartFrom.Visible = !isTestRunning;

                if (step.BtnTestSingle != null)
                    step.BtnTestSingle.Visible = !isTestRunning;
            }
        }


        // 添加步骤到UI (简化布局)
        private void AddStepToUI(TestStep step)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new Action(() => AddStepToUI(step)));
                return;
            }

            // 主容器面板
            var stepPanel = new Panel
            {
                Width = this.ClientSize.Width - 35,
                Height = 70, // 增加高度以容纳按钮
                Margin = new Padding(0, 0, 0, 5),
                BackColor = Color.White,
                BorderStyle = BorderStyle.FixedSingle,
                Tag = step
            };
            // 添加"从此开始"按钮
            var btnStartFrom = new Button
            {
                Text = "从此开始",
                Tag = step.Key, // 存储步骤Key
                Size = new Size(80, 25),
                Location = new Point(40, 35),
                FlatStyle = FlatStyle.Flat,
                BackColor = Color.LightSkyBlue,
                ForeColor = Color.White
            };
            btnStartFrom.Click += (s, e) =>
            {
                OnStepButtonClick?.Invoke(btnStartFrom.Tag.ToString(), TestType.StartFrom);
            };
            step.BtnStartFrom = btnStartFrom; // 保存按钮引用

            // 添加"单项测试"按钮
            var btnTestSingle = new Button
            {
                Text = "单项测试",
                Tag = step.Key, // 存储步骤Key
                Size = new Size(80, 25),
                Location = new Point(130, 35),
                FlatStyle = FlatStyle.Flat,
                BackColor = Color.LightSeaGreen,
                ForeColor = Color.White
            };
            btnTestSingle.Click += (s, e) =>
            {
                OnStepButtonClick?.Invoke(btnTestSingle.Tag.ToString(), TestType.TestSingle);
            };
            step.BtnTestSingle = btnTestSingle; // 保存按钮引用

            // 初始可见性设置
            btnStartFrom.Visible = !isTestRunning;
            btnTestSingle.Visible = !isTestRunning;

            // 状态图标
            var iconBox = new PictureBox
            {
                Image = GetStatusIcon(step.Status),
                SizeMode = PictureBoxSizeMode.CenterImage,
                Size = new Size(30, 30),
                Location = new Point(5, 2),
                BackColor = Color.Transparent
            };

            // 步骤名称标签 - 占据剩余空间
            var nameLabel = new Label
            {
                Text = step.Name,
                Font = new Font("Microsoft YaHei UI", 10, FontStyle.Regular),
                AutoSize = false,
                Location = new Point(40, 0),
                Size = new Size(stepPanel.Width - 45, 35),
                TextAlign = ContentAlignment.MiddleLeft,
                AutoEllipsis = true,
                Padding = new Padding(5, 0, 5, 0)
            };

            // 悬停效果
            stepPanel.MouseEnter += (s, e) =>
            {
                stepPanel.BackColor = Color.FromArgb(245, 245, 245);
            };

            stepPanel.MouseLeave += (s, e) =>
            {
                stepPanel.BackColor = Color.White;
            };

            // 添加控件
            stepPanel.Controls.Add(iconBox);
            stepPanel.Controls.Add(nameLabel);
            stepPanel.Controls.Add(btnStartFrom);
            stepPanel.Controls.Add(btnTestSingle);

            listContainer.Controls.Add(stepPanel);
            stepControls[step.Key] = stepPanel;
        }

        // 更新步骤UI
        private void UpdateStepUI(TestStep step)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new Action(() => UpdateStepUI(step)));
                return;
            }

            if (stepControls.TryGetValue(step.Key, out var stepPanel))
            {
                // 更新图标
                var iconBox = stepPanel.Controls[0] as PictureBox;
                if (iconBox != null)
                    iconBox.Image = GetStatusIcon(step.Status);

                // 更新名称标签
                var nameLabel = stepPanel.Controls[1] as Label;
                if (nameLabel != null)
                {
                    nameLabel.Text = step.Name;
                }
            }
        }

        // 获取状态图标
        private Image GetStatusIcon(StepStatus status)
        {
            switch (status)
            {
                case StepStatus.NotRun: return PendingIcon;
                case StepStatus.Waiting: return WaitingIcon;
                case StepStatus.Running: return RunningIcon;
                case StepStatus.Success: return SuccessIcon;
                case StepStatus.Failed: return FailedIcon;
                default: return PendingIcon;
            }
        }

        // 创建状态图标
        private static Image CreateStatusIcon(Color color)
        {
            var size = 20;
            var bmp = new Bitmap(30, 30); // 固定尺寸30x30

            using (var g = Graphics.FromImage(bmp))
            {
                g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
                g.Clear(Color.Transparent);

                // 绘制状态圆形
                using (var brush = new SolidBrush(color))
                {
                    g.FillEllipse(brush, 5, 5, size, size);
                }

                // 添加白色边框
                using (var pen = new Pen(Color.White, 1))
                {
                    g.DrawEllipse(pen, 5, 5, size, size);
                }

                // 添加状态符号
                if (color == SuccessColor)
                {
                    using (var pen = new Pen(Color.White, 2))
                    {
                        g.DrawLine(pen, 8, 15, 13, 20);
                        g.DrawLine(pen, 13, 20, 22, 10);
                    }
                }
                else if (color == FailedColor)
                {
                    using (var pen = new Pen(Color.White, 2))
                    {
                        g.DrawLine(pen, 8, 8, 22, 22);
                        g.DrawLine(pen, 22, 8, 8, 22);
                    }
                }
                else if (color == WaitingColor)
                {
                    using (var pen = new Pen(Color.White, 2))
                    {
                        g.DrawLine(pen, 15, 8, 15, 18);
                        g.FillEllipse(Brushes.White, 14, 20, 2, 2);
                    }
                }
            }

            return bmp;
        }

        // 重置所有步骤
        public void Reset()
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new Action(Reset));
                return;
            }

            steps.Clear();
            stepControls.Clear();
            listContainer.Controls.Clear();
        }
    }
}